/*
 * Copyright (c) 2022, Valaphee.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.valaphee.cran.node.input

import com.fasterxml.jackson.annotation.JsonFormat

/**
 * @author Kevin Ludwig
 */
@JsonFormat(shape = JsonFormat.Shape.NUMBER)
enum class Key(
    val scanCode: Int,
    val vkCode: Int = 0x00
) {
    None(0x00),
    A(0x04, 0x41),
    B(0x05, 0x42),
    C(0x06, 0x43),
    D(0x07, 0x44),
    E(0x08, 0x45),
    F(0x09, 0x46),
    G(0x0A, 0x47),
    H(0x0B, 0x48),
    I(0x0C, 0x49),
    J(0x0D, 0x4A),
    K(0x0E, 0x4B),
    L(0x0F, 0x4C),
    M(0x10, 0x4D),
    N(0x11, 0x4E),
    O(0x12, 0x4F),
    P(0x13, 0x50),
    Q(0x14, 0x51),
    R(0x15, 0x52),
    S(0x16, 0x53),
    T(0x17, 0x54),
    U(0x18, 0x55),
    V(0x19, 0x56),
    W(0x1A, 0x57),
    X(0x1B, 0x58),
    Y(0x1C, 0x59),
    Z(0x1D, 0x5A),
    `1`(0x1E, 0x31),
    `2`(0x1F, 0x32),
    `3`(0x20, 0x33),
    `4`(0x21, 0x34),
    `5`(0x22, 0x35),
    `6`(0x23, 0x36),
    `7`(0x24, 0x37),
    `8`(0x25, 0x38),
    `9`(0x26, 0x39),
    `0`(0x27, 0x3),
    Enter(0x28, 0x0D),
    Escape(0x29, 0x1B),
    Backspace(0x2A, 0x08),
    Tab(0x2B, 0x09),
    Space(0x2C, 0x20),
    OemMinus(0x2D, 0xBD),
    OemPlus(0x2E, 0xBB),
    Oem4(0x2F, 0xDB),
    Oem6(0x30, 0xDD),
    Oem5(0x31, 0xDC),
    Oem1(0x33, 0xBA),
    Oem7(0x34, 0xDE),
    Oem3(0x35, 0xC0),
    OemComma(0x36, 0xBC),
    OemPeriod(0x37, 0xBE),
    Oem2(0x38, 0xBF),
    Capslock(0x39, 0x14),
    F1(0x3A, 0x70),
    F2(0x3B, 0x71),
    F3(0x3C, 0x72),
    F4(0x3D, 0x73),
    F5(0x3E, 0x74),
    F6(0x3F, 0x75),
    F7(0x40, 0x76),
    F8(0x41, 0x77),
    F9(0x42, 0x78),
    F10(0x43, 0x79),
    F11(0x44, 0x7A),
    F12(0x45, 0x7B),
    SysRq(0x46),
    ScrollLock(0x47, 0x91),
    Pause(0x48, 0x13),
    Insert(0x49, 0x2D),
    Home(0x4A, 0x24),
    PageUp(0x4B, 0x21),
    Delete(0x4C, 0x2E),
    End(0x4D, 0x23),
    PageDown(0x4E, 0x22),
    Right(0x4F, 0x27),
    Left(0x50, 0x25),
    Down(0x51, 0x28),
    Up(0x52, 0x26),
    NumLock(0x53, 0x90),
    NumPadDivide(0x54, 0x6F),
    NumPadMultiply(0x55, 0x6A),
    NumPadSubtract(0x56, 0x6D),
    NumPadAdd(0x57, 0x6B),
    NumPadEnter(0x58),
    NumPad1(0x59, 0x61),
    NumPad2(0x5a, 0x62),
    NumPad3(0x5b, 0x63),
    NumPad4(0x5c, 0x64),
    NumPad5(0x5d, 0x65),
    NumPad6(0x5e, 0x66),
    NumPad7(0x5f, 0x67),
    NumPad8(0x60, 0x68),
    NumPad9(0x61, 0x69),
    NumPad0(0x62, 0x60),
    NumPadDecimal(0x63, 0x6E),
    Oem102(0x64, 0xE2),
    LeftControl(0xE0, 0xA2),
    LeftShift(0xE1, 0xA0),
    LeftAlt(0xE2, 0xA4),
    LeftMeta(0xE3),
    RightControl(0xE4, 0xA3),
    RightShift(0xE5, 0xA1),
    RightAlt(0xE6, 0xA5),
    RightMeta(0xE7);

    companion object {
        private val byVkCode = values().associateBy { it.vkCode }

        fun byVkCode(vkCode: Int) = byVkCode[vkCode]
    }
}
